/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes;

import org.wso2.registry.Artifact;
import org.wso2.registry.RegistryException;
import org.wso2.registry.jdbc.ConnectionFactory;
import org.wso2.registry.jdbc.dao.VersionedArtifactDAO;
import org.wso2.registry.jdbc.utils.AuthorizationUtil;
import org.wso2.usermanager.Realm;

/**
 * Base class of all media type handlers. It defines the three basic methods to be implemented by
 * all media type handlers. This is initialized with the objects required for handling media type
 * specific operations, so that extending media type handlers can easily access them. One such
 * object is the instance of the MediaTypeManager.
 */
public abstract class MediaTypeHandler {

    /**
     * Connection factory to obtaining database connections. Media type handlers need to access the
     * database for performing almost all the tasks.
     */
    protected ConnectionFactory connectionFactory;

    /**
     * User manager realm, which can be used to set permissions on resources. Media type handlers
     * may want to add other resources (e.g. imports) than the originally given resource. So it is
     * necessary to allow media type handler to set permissions for the resources it is dealing
     * with.
     */
    protected Realm realm;

    /**
     * While processing media types, handlers may want to access resources with other media types.
     * Then they can just call the appropriate methods of the MediaTypeManager for doing that.
     *
     * e.g. "text" mediatype handler may just want to change the content of the resource from String to
     * byte[] before PUT. So that users can set Strings as the content of text resources. But after
     * changing that, text handler want PUT the resource using normal process. In that case, text
     * handler can execute the put(...) method of the DefaultMediaTypeHandler after doing the relevent
     * changes to the resource.
     */
    protected MediaTypeManager mediaTypeManager;

    /**
     * ArtifactDAO for directly accessing resources.
     */
    //protected ArtifactDAO artifactDAO = new ArtifactDAO();
    protected VersionedArtifactDAO artifactDAO = new VersionedArtifactDAO();

    /**
     * Some common authorization tasks are implemented in this util.
     */
    protected AuthorizationUtil authorizationUtil = new AuthorizationUtil();

    public MediaTypeHandler(ConnectionFactory connectionFactory,
                            Realm realm,
                            MediaTypeManager mediaTypeManager) {

        this.connectionFactory = connectionFactory;
        this.realm = realm;
        this.mediaTypeManager = mediaTypeManager;
    }

    /**
     * Processes the GET action of the media type.
     *
     * @param path Path of the resource
     *
     * @param rawArtifact Raw artifact at the given path, which is retrieved directly from the
     * database. This can be used by media type handlers, which need to alter contents of the
     * raw artifact, etc.
     *
     * @return Artifact instance if the media type handler processes the GET action. If artifact is
     * returned it should perform all tasks to fulfill the GET request. Returns null if the
     * media type handler does not process the GET action. If null is returned, get(...) method of
     * the DefaultMediaTypeHandler will be invoked.
     *
     * @throws RegistryException
     */
    public abstract Artifact get(String path, Artifact rawArtifact) throws RegistryException;

    /**
     * Processes the PUT action of the media type.
     *
     * @param path to put the resource.
     *
     * @param artifact to put.
     *
     * @return true if the media type handler processes the PUT action. If true is returned it
     * should perform all tasks to fulfill the PUT request. Returns false if the media type handler
     * does not process the PUT action. If false is returned, put(...) method of the
     * DefaultMediaTypeHandler will be invoked.
     *
     * @throws RegistryException
     */
    public abstract boolean put(String path, Artifact artifact) throws RegistryException;

    /**
     * Processes the DELETE action of the media type.
     *
     * @param path path of the resource to be deleted.
     *
     * @return true if the media type handler processes the DELETE action. If true is returned it
     * should perform all tasks to fulfill the DELETE request. Returns false if the media type handler
     * does not process the DELETE action. If false is returned, delete(...) method of the
     * DefaultMediaTypeHandler will be invoked.
     *
     * @throws RegistryException
     */
    public abstract boolean delete(String path) throws RegistryException;
}
